@@ -1024,18 +1024,19 @@ class AddGaussianNoise(object):
         self.mean = mean
         
     def __call__(self, tensor):
-        return tensor + torch.randn(tensor.size()) * self.std + self.mean
+        rand_std = random.uniform(0, self.std)
+        return tensor + torch.randn(tensor.size()) * rand_std + self.mean
     
     def __repr__(self):
         return self.__class__.__name__ + '(mean={0}, std={1})'.format(self.mean, self.std)
 
 
-def CreateImagesetLoaders(s3, s3def, src, dest, bucket = None, width=256, height=256, batch_size = 2, shuffle=True, 
+def CreateImagenetLoaders(s3, s3def, src, dest, bucket = None, width=256, height=256, batch_size = 2, shuffle=True, 
                       num_workers=0, cuda = True, timeout=0, loaders = None, 
                       image_transform=None, label_transform=None, 
                       normalize=True, flipX=True, flipY=False, 
                       random_seed = None, numTries=3,
-                      rotate=3, scale_min=0.75, scale_max=1.25, offset=0.1, augment_noise=0.0):
+                      rotate=3, scale_min=0.75, scale_max=1.25, offset=0.1, augment_noise=1.0):
 
     if not bucket:
         bucket = s3def['sets']['dataset']['bucket']
@@ -1058,7 +1059,7 @@ def CreateImagesetLoaders(s3, s3def, src, dest, bucket = None, width=256, height
                 translate=(offset, offset), 
                 scale=(scale_min, scale_max), 
                 interpolation=transforms.InterpolationMode.BILINEAR)) 
-        transform_list.appendtransforms.RandomCrop( (width, height), padding=None, pad_if_needed=True, fill=0, padding_mode='constant'))
+        transform_list.append(transforms.RandomCrop( (width, height), padding=None, pad_if_needed=True, fill=0, padding_mode='constant'))
         transform_list.append(transforms.ToTensor())
         transform_list.append(transforms.Normalize((0.0, 0.0, 0.0), (1.0, 1.0, 1.0))) # Imagenet mean and standard deviation
         if augment_noise > 0.0:
@@ -1106,15 +1107,27 @@ def main(args):
 
     s3, creds, s3def = Connect(args.credentails)
 
+    parameters = ReadDict(args.test_config)
+
     if args.test_dataset:
-        loaders = CreateImagesetLoaders(s3, s3def, 
+        loaders = CreateImagenetLoaders(s3, s3def, 
                                     args.obj_src, 
                                     args.destination, 
                                     width=args.width, 
                                     height=args.height, 
                                     batch_size=args.batch_size, 
                                     num_workers=args.num_workers,
-                                    cuda = args.cuda)
+                                    cuda = args.cuda,
+                                    flipX=parameters['imagenet']['flipX'], 
+                                    flipY=parameters['imagenet']['flipY'], 
+                                    rotate=parameters['imagenet']['rotate'], 
+                                    scale_min=parameters['imagenet']['scale_min'], 
+                                    scale_max=parameters['imagenet']['scale_max'], 
+                                    offset=parameters['imagenet']['offset'], 
+                                    augment_noise=parameters['imagenet']['augment_noise'])
+
+        parameters['imagenet']['test_path']=os.path.join(parameters['imagenet']['test_path'], '') # Add trailing slash if not present
+        os.makedirs(parameters['imagenet']['test_path'], exist_ok=True)
 
         for loader in tqdm(loaders, desc="Loader"):
             for i, data in tqdm(enumerate(loader['dataloader']), 
@@ -1127,6 +1140,16 @@ def main(args):
                 assert(sample.shape[3] == args.width)
                 assert(target is not None)
 
+                sample =  sample.permute(0, 2, 3, 1) # Change to batch, height, width, channel for rendering
+                sample_max = sample.max()
+                sample_min = sample.min()
+                if sample_max > sample_min:
+                    for j, image in enumerate(sample):
+                        image = 255*(image - sample_min)/(sample_max-sample_min) # Convert to RGB color rane
+                        image = image.cpu().numpy().astype(np.uint8)
+                        write_path = '{}{}{:03d}{:03d}.png'.format(parameters['imagenet']['test_path'], loader['set'], i,j)            
+                        cv2.imwrite(write_path,image)
+
             if args.num_images is not None and args.num_images > 0 and i >= args.num_images:
                 print ('test_iterator complete')
                 break
@@ -1144,7 +1167,7 @@ def parse_arguments():
     parser.add_argument('-debug_address', type=str, default='0.0.0.0', help='Debug port')
     parser.add_argument('-credentails', type=str, default='creds.yaml', help='Credentials file.')
     parser.add_argument('-num_images', type=int, default=0, help='Number of images to display')
-    parser.add_argument('-num_workers', type=int, default=25, help='Data loader workers')
+    parser.add_argument('-num_workers', type=int, default=0, help='Data loader workers')
     parser.add_argument('-batch_size', type=int, default=4, help='Dataset batch size')
     parser.add_argument('-i', action='store_true', help='True to test iterator')
     parser.add_argument('-test_iterator', type=bool, default=False, help='True to test iterator')
